/*
 * Generation d'ensembles de points (surfaces classiques).
 * Ou bien lecture d'un fichier (stdin).
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "structures.h"
#include "genere.h"


/*
 * Cette fonction lit l'entree standard qui doit etre de la forme:
 *
 * topax% less /dl_32/SPI/GRILLE.DAT             
 * 77361 241 321 0
 *   0.000000    0.000000    -16.304428 0
 *   0.000000    0.509449    -16.285797 0
 *   0.000000    1.018897    -16.324158 0
 *   0.000000    1.528346    -16.324532 0
 *   0.000000    2.037794    -16.325758 0
 * ...
 * Et sort le tableau de points correspondant.
 */

pt3d *gen_file ( unsigned int *nmbpts, FILE *file)
{
    int a, b, c, d, i, j;
    float x, y, z;
    pt3d *tableau;
    
    if (4 == fscanf (file, "%d%d%d%d", nmbpts, &a, &b, &c))
    {
	tableau = (pt3d *) calloc (*nmbpts, sizeof(pt3d));
        for (i=j=0; i<*nmbpts; i++)
	{
	    if (4 != fscanf (file, "%f%f%f%d", &x, &y, &z, &d))
	      {
		printf("Erreur de lecture du fichier\n");
		exit(-1);
	      };
	    if (d != 0)
	    {
		printf("Point rejete.\n");
		-- *nmbpts;
	    }
	    else
	    {
		tableau[j].x = x;
		tableau[j].y = y;
		tableau[j].z = z;
		++j;
	    };
	};
    }
    else
      {
	printf("erreur de lecture du fichier\n");
	exit(-1);
      };
    return tableau;
}


/*
 * Cette fonction rempli un tableau de points situes sur un paraboloide
 * (ou un hyperboloide a une nappe), d'equation z = a.x^2 + b.y^2.
 * Les points sont choisis aleatoirement, avec les conditions
 * d'intervalle sur x et y.
 */

pt3d *gen_para (    int nmbpts,
		    double xmin,
		    double xmax,
		    double ymin,
		    double ymax,
		    double a,
		    double b)
{
    int i;
    pt3d *tableau = (pt3d *) calloc (nmbpts, sizeof(pt3d));

    if (tableau == NULL) {
	printf("Erreur d'allocation memoire.\n");
	exit(-1);
    }
    else {
        for (i=0; i<nmbpts; i++) {
	    tableau[i].x = xmin + ( (xmax-xmin) * (rand() % 32768) ) / 32768;
	    tableau[i].y = ymin + ( (ymax-ymin) * (rand() % 32768) ) / 32768;
	    tableau[i].z = a * (tableau[i].x) * (tableau[i].x) +
		    	   b * (tableau[i].y) * (tableau[i].y);
	};
    };

    return tableau;
}

pt3d *gen_sph ( int nmbpts,
		double x,
		double y,
		double z,
		double ray)
{
    int i;
    double dist;
    pt3d *tableau = (pt3d *) calloc (nmbpts, sizeof(pt3d));

    if (tableau == NULL) {
	printf("Erreur d'allocation memoire.\n");
	exit(-1);
    }
    else {
      for (i=0; i<nmbpts; i++) {
	tableau[i].x = ((rand() % 32768) / 32768.0) / 5;
	tableau[i].y = (rand() % 32768 - 16384) / 32768.0;
	tableau[i].z = (rand() % 32768 - 16384) / 32768.0;
	dist = sqrt(SQR(tableau[i].x) + SQR(tableau[i].y) + SQR(tableau[i].z));
	tableau[i].x = x + ray * tableau[i].x /dist;
	tableau[i].y = y + ray * tableau[i].y /dist;
	tableau[i].z = z + ray * tableau[i].z /dist;
      };
    };

    return tableau;
}


/*
 * Cette fonction rempli un tableau de points situes sur un cube.
 * (seulement 3 faces)
 */

pt3d *gen_cube (    int nmbpts,
		    double cote)
{
    int i;
    double interim;
    pt3d *tableau = (pt3d *) calloc (nmbpts, sizeof(pt3d));

    if (tableau == NULL) {
	printf("Erreur d'allocation memoire.\n");
	exit(-1);
    }
    else
    {
        for (i=0; i<nmbpts; i++)
	{
	    tableau[i].x = (cote * (rand() % 32768)) / 32768;
	    tableau[i].y = (cote * (rand() % 32768)) / 32768;
	    tableau[i].z = (cote * (rand() % 32768)) / 32768;

	    interim = (1.0 * (rand() % 32768)) / 32768;
	    if (interim < (1./3.)) {
		tableau[i].x /= 500;
	    } else {
		if (interim > (2./3.)) {
			tableau[i].y /= 500;
		} else	tableau[i].z /= 500;
	    };
	};
    };
    return tableau;
}


/*
 * Cette fonction rempli un tableau de points situes sur un cube.
 * (les 6 faces)
 */

pt3d *gen_fullcube ( int nmbpts,
		     double cote)
{
    int i;
    double interim;
    pt3d *tableau = (pt3d *) calloc (nmbpts, sizeof(pt3d));

    if (tableau == NULL) {
	printf("Erreur d'allocation memoire.\n");
	exit(-1);
    }
    else
    {
        for (i=0; i<nmbpts; i++)
	{
	    tableau[i].x = (cote * (rand() % 32768)) / 32768;
	    tableau[i].y = (cote * (rand() % 32768)) / 32768;
	    tableau[i].z = (cote * (rand() % 32768)) / 32768;

	    interim = (1.0 * (rand() % 32768)) / 32768;
	    if (interim < (1./3.)) {
		tableau[i].x /= 500;
		if (interim < (1./6.)) tableau[i].x = cote - tableau[i].x;
	    } else {
		if (interim > (2./3.)) {
		    tableau[i].y /= 500;
		    if (interim > (5./6.)) tableau[i].y = cote - tableau[i].y;
		} else {
		    tableau[i].z /= 500;
		    if (interim > (1./2.)) tableau[i].z = cote - tableau[i].z;
		};
	    };
	};
    };
    return tableau;
}
