
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "structures.h"
#include "vect_normal.h"

/*
 * Cette fonction prend en parametre les coefficients d'une matrice
 * symetrique:
 *
 * a d e
 * d b f     On sait que a, b et c sont positifs.
 * e f c
 *
 * et calcule un vecteur unitaire correspondant a la plus petite valeur
 * propre.
 * 
 * Il y a plusieurs methodes: la premiere consiste a resoudre le polynome
 * caracteristique, soit directement, soit par Newton, et de calculer un
 * vecteur du noyau ensuite.
 *
 * On peut utiliser, a la place, la methode decrite dans l'annexe du
 * rapport de stage de DEA de Nazih Saade.
 *
 */

    /*
     * Fonction qui calcule les coeffs de la matrice
     * symetrique correspondant aux points voisins, 
     * d'un point donne, et qui appelle vect_normal().
     * C'est ici qu'on choisit la methode exacte pour
     * determiner la normale.
     * 
     * On calcule ici les coefficients de la matrice d'inertie.
     */
     
vect *mat (pt3d *point, str_mem *tab)
{
    unsigned int i;
    double dx, dy, dz;
    double a, b, c, d, e, f;
    double coeff;
    
    for (i=0, a=b=c=d=e=f=0; i<tab->current; i++)
	if (point != tab->debut[i])
	{
	    coeff = 1/( point->x * point->x +
			point->y * point->y +
			point->z * point->z );
	    dx = point->x - tab->debut[i]->x;
	    dy = point->y - tab->debut[i]->y;
	    dz = point->z - tab->debut[i]->z;

	    a += dx * dx * coeff;
	    b += dy * dy * coeff;
	    c += dz * dz * coeff;
	    d += dx * dy * coeff;
	    e += dx * dz * coeff;
	    f += dy * dz * coeff;
	};

    a /= tab->current;
    b /= tab->current;
    c /= tab->current;
    d /= tab->current;
    e /= tab->current;
    f /= tab->current;

    return vect_normal (a, b, c, d, e, f, 0.00000001);
}
    
    
    /* Methode dite des "iterees inverse". */
     
vect *vect_normal ( double a, double b,
		    double c, double d,
		    double e, double f,
		    double epsilon)
{
    double det = 2*d*e*f + a*b*c - c*d*d - b*e*e - a*f*f; 
    double a1  = (b*c - f*f) / det;
    double b1  = (a*c - e*e) / det;
    double c1  = (a*b - d*d) / det;
    double d1  = (e*f - d*c) / det;
    double e1  = (d*f - e*b) / det;
    double f1  = (d*e - a*f) / det;
    double a2  = SQR(a1) + SQR(d1) + SQR(e1);
    double b2  = SQR(b1) + SQR(d1) + SQR(f1);
    double c2  = SQR(c1) + SQR(f1) + SQR(e1);
    double d2  = (a1 + b1)*d1 + e1*f1;
    double e2  = (a1 + c1)*e1 + d1*f1;
    double f2  = (c1 + b1)*f1 + e1*d1;
    double dist = DBL_MAX;

    vect v0 = {1, 0, 0};
    vect *v1 = (vect *) ( malloc (sizeof (vect)) );

    if (v1 == NULL) {
	printf("Erreur d'allocation memoire dans vect_normal().\n");
	exit(-1);
    };

    /*
     * On peut factoriser (multiplier par A^-2 d'un seul coup...)
     */

    while (dist > epsilon)
    {

      /* On peut factoriser (multiplier par A^-2 d'un seul coup...) */

	v1->x = a2 * v0.x + d2 * v0.y + e2 * v0.z;
	v1->y = d2 * v0.x + b2 * v0.y + f2 * v0.z;
	v1->z = e2 * v0.x + f2 * v0.y + c2 * v0.z;

	dist = sqrt(SQR(v1->x) + SQR(v1->y) + SQR(v1->z));

	v1->x /= dist;
	v1->y /= dist;
	v1->z /= dist;
	
	v0.x -= v1->x;
	v0.y -= v1->y;
	v0.z -= v1->z;
	
	dist = SQR(v0.x) + SQR(v0.y) + SQR(v0.z);

	v0.x = v1->x;
	v0.y = v1->y;
	v0.z = v1->z;
    };

    return v1;
}
